<?php
/*
Plugin Name: EngageBay Add-on For Contact Form 7
Plugin URI: https://www.engagebay.com/contact-form7
Description: EngageBay Add-on For Contact Form 7 integration plugin for Contact Form 7 forms. Sync form entries to my website easily.
Author: EngageBay
Version: 1.7.2
Tested up to: 6.0
Author URI: https://engagebay.com/

PREFIX: cf7ebi (Contact Form 7 Engagebay Forms Integration)

*/

//check to make sure contact form 7 is installed and active
include_once ABSPATH . 'wp-admin/includes/plugin.php';

if (is_plugin_active('contact-form-7/wp-contact-form-7.php')) {
    function cf7ebi_root_url($append = false)
    {
        $base_url = plugin_dir_url(__FILE__);

        return $append ? $base_url . $append : $base_url;
    }

    function cf7ebi_root_dir($append = false)
    {
        $base_dir = plugin_dir_path(__FILE__);

        return $append ? $base_dir . $append : $base_dir;
    }

    include_once cf7ebi_root_dir('inc/constants.php');

    function cf7ebi_enqueue($hook)
    {
        if (!strpos($hook, 'wpcf7')) {
            return;
        }

        wp_enqueue_style(
            'cf7ebi-styles',
            cf7ebi_root_url('assets/css/styles.css'),
            false,
            CF7EBI_VERSION
        );

        wp_enqueue_script(
            'cf7ebi-scripts',
            cf7ebi_root_url('assets/js/scripts.js'),
            array('jquery'),
            CF7EBI_VERSION
        );
    }
    add_action('admin_enqueue_scripts', 'cf7ebi_enqueue');

    function cf7ebi_engagebay_css()
    {
        wp_enqueue_style('engagebay-marketing-software', cf7ebi_root_url('assets/css/styles.css'));
    }

    function cf7ebi_engagebay_refresh()
    {
        if (isset($_POST['name'])) {
            wp_enqueue_script('engagebay_cf7ebi_refresh', cf7ebi_root_url('assets/js/refresh.js'));
        }
    }

    add_action('admin_menu', 'wpcf7_engagebay_admin_menu', 10);

    function wpcf7_engagebay_admin_menu()
    {
        add_submenu_page(
            'wpcf7',
            __('EngageBay', 'contact-form-7'),
            __('EngageBay', 'contact-form-7'),
            'administrator',
            'engagebay-cf7-settings',
            'cf7ebi_login_settings'
        );
    }

    add_action('plugins_loaded', 'wpcf7_engagebay_plugin_override');
    function wpcf7_engagebay_plugin_override()
    {
        if (isset($_GET['code'])) {

            require_once 'vendor/autoload.php';

            $provider = new \League\OAuth2\Client\Provider\GenericProvider([
                'clientId'                => '765095083631-uo5n5br03hl68ora8oohooeu6j0n1dkv.apps.googleusercontent.com',    // The client ID assigned to you by the provider
                'clientSecret'            => 'zAqrJN80mWUKhD5soIZk95u6',   // The client password assigned to you by the provider
                'redirectUri'             => 'https://app.engagebay.com/oauth2callback',
                'urlAuthorize'            => 'https://accounts.google.com/o/oauth2/auth',
                'urlAccessToken'          => 'https://www.googleapis.com/oauth2/v4/token',
                'urlResourceOwnerDetails' => 'https://www.googleapis.com/oauth2/v3/userinfo',
                'scopes' => 'email,profile'
            ]);

            try {
                // Try to get an access token using the authorization code grant.
                $accessToken = $provider->getAccessToken('authorization_code', [
                    'code' => $_GET['code']
                ]);

                $resourceOwner = $provider->getResourceOwner($accessToken);

                $user = $resourceOwner->toArray();

                $api_url = 'https://app.engagebay.com//dev/api/panel/api';
                $response = wp_remote_get(
                    $api_url,
                    array(
                        'timeout' => 40,
                        'method' => 'GET',
                        'sslverify' => true,
                        'headers' => array('Authorization' => 'EngageBayDevAPI!@#', 'ebwhitelist' => true, 'Accept' => 'application/json;ver=1.0', 'Content-Type' => 'application/json; charset=UTF-8', 'email' => $user['email']),
                    )
                );


                if (version_compare(PHP_VERSION, '5.4.0', '>=') && !(defined('JSON_C_VERSION') && PHP_INT_SIZE > 4)) {
                    $result = json_decode($response['body'], false, 512, JSON_BIGINT_AS_STRING);
                } else {
                    $result = json_decode($response['body'], false);
                }

                if ($result && $result->rest_API_Key) {
                    if (get_option('cf7ebi_settings_rest_api') !== false) {
                        update_option('cf7ebi_settings_rest_api', $result->rest_API_Key);
                    } else {
                        add_option('cf7ebi_settings_rest_api', $result->rest_API_Key);
                    }

                    $domain = explode(".", $result->version_url)[0];
                    $domain = str_replace("https://", "", $domain);

                    if (get_option('cf7ebi_settings_domain') !== false) {
                        update_option('cf7ebi_settings_domain', $domain);
                    } else {
                        add_option('cf7ebi_settings_domain', $domain);
                    }

                    if (get_option('cf7ebi_settings_js_api') !== false) {
                        update_option('cf7ebi_settings_js_api', $result->js_API_Key);
                    } else {
                        add_option('cf7ebi_settings_js_api', $result->js_API_Key);
                    }

                    if (get_option('cf7ebi_settings_email') !== false) {
                        update_option('cf7ebi_settings_email', $user['email']);
                    } else {
                        add_option('cf7ebi_settings_email', $user['email']);
                    }
                }
            } catch (\League\OAuth2\Client\Provider\Exception\IdentityProviderException $e) {

                // Failed to get the access token or user details.
                exit($e->getMessage());
            }
        }
    }

    function cf7ebi_login_settings()
    {
        if (isset($_GET['tab']) && $_GET['tab'] == 'register') {
            cf7ebi_registration_page();

            return;
        }

        cf7ebi_engagebay_css();
        $deprecated = null;
        $autoload = 'no';

        if (isset($_POST['email'])) {
            $email = sanitize_email($_POST['email']);
        }
        if (isset($email) && $email) {
            $email = sanitize_email($_POST['email']);
            @$rest_api = sanitize_text_field($_POST['rest_api']);
            $password = sanitize_text_field($_POST['password']);
            $api_url = 'https://app.engagebay.com/rest/api/login/get-domain';
            $request = wp_remote_post($api_url, array(
                'method' => 'post',
                'body' => array('email' => $email, 'password' => $password, 'source' => 'WORDPRESS'),
                'headers' => array('ebwhitelist' => true),
            ));
            $result = wp_remote_retrieve_body($request);
            if (!is_wp_error($result)) {
                if (version_compare(PHP_VERSION, '5.4.0', '>=') && !(defined('JSON_C_VERSION') && PHP_INT_SIZE > 4)) {
                    $result = json_decode($result, true, 512, JSON_BIGINT_AS_STRING);
                } else {
                    $result = json_decode($result, true);
                }
            }

            if (isset($result['type']) && $result['type'] == 'ACCOUNT_NOT_ACTIVATED') {
                $errors = 'Your account is not activated. A confirmation email was sent to ' . $email;
            }

            $domain = isset($result['domain_name']) ? $result['domain_name'] : '';
            $js_api = isset($result['api_key']['js_API_Key']) ? $result['api_key']['js_API_Key'] : '';
            $rest_api = isset($result['api_key']['rest_API_Key']) ? $result['api_key']['rest_API_Key'] : '';
            if ($domain == '' && $rest_api == '' && !isset($errors)) {
                $errors = 'Invalid details. Please provide valid details and try again';
            } elseif (!isset($errors)) {
                if (get_option('cf7ebi_settings_password') !== false) {
                    update_option('cf7ebi_settings_password', $password);
                } else {
                    add_option('cf7ebi_settings_password', $password, $deprecated, $autoload);
                }

                if (get_option('cf7ebi_settings_rest_api') !== false) {
                    update_option('cf7ebi_settings_rest_api', $rest_api);
                } else {
                    add_option('cf7ebi_settings_rest_api', $rest_api, $deprecated, $autoload);
                }

                if (get_option('cf7ebi_settings_domain') !== false) {
                    update_option('cf7ebi_settings_domain', $domain);
                } else {
                    add_option('cf7ebi_settings_domain', $domain, $deprecated, $autoload);
                }

                if (get_option('cf7ebi_settings_js_api') !== false) {
                    update_option('cf7ebi_settings_js_api', $js_api);
                } else {
                    add_option('cf7ebi_settings_js_api', $js_api, $deprecated, $autoload);
                }

                if (get_option('cf7ebi_settings_email') !== false) {
                    update_option('cf7ebi_settings_email', $email);
                } else {
                    add_option('cf7ebi_settings_email', $email, $deprecated, $autoload);
                }

                $sucesserrors = 'SucessFully Verified the domain';
            }
        }
        $rest_api = (sanitize_text_field(get_option('cf7ebi_settings_rest_api')));
        $domain = (sanitize_text_field(get_option('cf7ebi_settings_domain')));
        $email = (sanitize_email(get_option('cf7ebi_settings_email')));
        if (isset($_GET['tab'])) {
            $tab = $_GET['tab'];
        } ?>
        <div class="be-wrapper be-login">
            <div class="be-content">
                <div class="main-content container-fluid">
                    <?php if (!isset($sucesserrors) && ($email == '' || (isset($tab) && $tab == 'edit') || $rest_api == '')) {
                    ?>

                        <div class="splash-container">
                            <div class="panel">
                                <?php if (isset($errors) && $errors) {
                                    echo '<div id="error_message">' . $errors . '<br/></div>';
                                } ?>
                                <?php if (isset($sucesserrors) && $sucesserrors) {
                                    //echo '<div id="success_message">' . $sucesserrors . '<br/></div>';
                                    wp_redirect(cf7ebi_logged_in_page());
                                } elseif (get_option('engagebay_rest_api') !== false) {
                                    if (!isset($tab)) {
                                        echo '<div id="success_message">You are Connected with EngageBay</div>';
                                    }
                                } ?>
                                <div class="panel-heading text-center">
                                    <?php echo "<img src='" . cf7ebi_root_url('assets/images/engagebay.png') . "' width='170' title='EngageBay logo'/>"; ?>
                                    <h4>EngageBay Login</h4>
                                </div>
                                <div class="panel-body">
                                    <form name="loginForm" id="loginForm" class="" action="" method="post">

                                        <div class="form-group">
                                            <div class="">
                                                <input class="form-control" type="email" pattern="[a-z0-9._%+-]+@[a-z0-9.-]+\.[a-z]{2,20}$" name="email" autocomplete="off" placeholder="Work Email" value="<?php echo $email; ?>" required>
                                            </div>
                                        </div>
                                        <div class="form-group">
                                            <div class="">
                                                <input class="form-control" type="password" name="password" placeholder="Password" required>
                                            </div>
                                        </div>
                                        <div class="form-group">
                                            <div class="xs-pt-20 md-pt-20">
                                                <div class="login-submit xs-m-0 md-p-0">
                                                    <button class="btn btn-success btn-xl" type="submit">LOGIN
                                                        <?php echo "<img src='" . plugins_url('/images/loader.gif', __FILE__) . "' 
                            style='display: none;' id='gif' title='Landing Pages'/>"; ?>
                                                </div><br />
                                                <div class="btn btn-success btn-xl gsuite-btn-div">
                                                    <a><img class="gsuite-img" src="https://d2p078bqz5urf7.cloudfront.net/cloud/dev/assets/img/google-favicon.png" /></a>
                                                    <a class="gsuite-btn-text" href="https://app.engagebay.com/oauth?wordpressPlugin=yes&wordpressPluginURL=<?php echo admin_url('admin.php?page=engagebay-cf7-settings'); ?>">SIGN IN WITH G SUITE</a>
                                                </div>
                                            </div>
                                        </div>

                                    </form>
                                </div>
                            </div>

                            <div class="splash-footer">
                                Forgot <a href="https://app.engagebay.com/forgot-password" target="_blank" class="text-info">Password?</a>
                            </div>
                            <div class="alert text-center">
                                <div>
                                    <!-- <div>Don't have an account? <a href="?page=engagebay-cf7-settings&tab=register" ><?php _e('Sign Up', 'sandbox'); ?></a></div> -->
                                    <div>Don't have an account? <a target="_blank" href="https://app.engagebay.com/signup"><?php _e('Sign Up', 'sandbox'); ?></a></div>
                                </div>
                            </div>


                        </div>
                </div>

            <?php
                    } elseif ($email != false) {
                        wp_redirect(cf7ebi_logged_in_page()); ?>

            </div>


        <?php
                    } else {
                        //wp_redirect(cf7ebi_logged_in_page());
                    } ?>
        </div>
        </div>
        </div>

    <?php
    }

    function cf7ebi_logged_in_page()
    {
    ?>
        <div class="be-wrapper be-login">
            <div class="be-content">
                <div class="main-content container-fluid">
                    <div class="splash-container">
                        <div class="panel">
                            <div class="panel-heading text-center">
                                <?php echo "<img src='" . cf7ebi_root_url('assets/images/engagebay.png') . "' width='170' title='EngageBay logo'/>"; ?>
                                <h4>You are Connected to EngageBay</h4>
                            </div>
                            <div class="panel-body">
                                <form name="loginForm" id="loginForm" class="" action="" method="post">

                                    <div class="form-group">
                                        <div class="">
                                            <label>Domain </label>
                                            <input class="form-control" type="email" pattern="[a-z0-9._%+-]+@[a-z0-9.-]+\.[a-z]{2,5}$" name="email" autocomplete="off" placeholder="Work Email" value="<?php echo get_option('cf7ebi_settings_domain'); ?>" disabled required>
                                        </div>
                                    </div>
                                    <div class="form-group">
                                        <div class="">
                                            <label>Email </label>
                                            <input class="form-control" type="email" pattern="[a-z0-9._%+-]+@[a-z0-9.-]+\.[a-z]{2,5}$" name="email" autocomplete="off" placeholder="Work Email" value="<?php echo get_option('cf7ebi_settings_email'); ?>" disabled required>
                                        </div>
                                    </div>

                                    <div class="form-group">
                                        <div class="xs-pt-20 md-pt-20">
                                            <div class="login-submit xs-m-0 md-p-0">
                                                <a style="padding:0px" class="btn btn-success btn-xl" href="admin.php?page=engagebay-cf7-settings&tab=edit">Login as different user</a>
                                                <?php echo "<img src='" . cf7ebi_root_url('assets/images/loader.gif') . "' 
                       style='display: none;' id='gif' title='Landing Pages'/>"; ?>
                                            </div>
                                        </div>
                                    </div>

                                </form>
                            </div>
                        </div>

                        <?php
                        //echo get_option('cf7ebi_settings_domain');
                        if (get_option('cf7ebi_settings_domain') === false) {
                        ?>
                            <div class="splash-footer">
                                Forgot <a href="https://app.engagebay.com/forgot-password" target="_blank" class="text-info">Password?</a>
                            </div>
                            <div class="alert text-center">
                                <div>
                                    <div>Don't have an account? <a href="?page=engagebay-cf7-settings&tab=register"><?php _e('Sign Up', 'sandbox'); ?></a>
                                    </div>
                                </div>
                            </div>
                        <?php
                        } ?>
                    </div>
                </div>
            </div>
        </div>
    <?php
    }

    function cf7ebi_registration_page()
    {
        cf7ebi_engagebay_css();
        if (isset($_POST['email'])) {
            $email = sanitize_email($_POST['email']);
        }
        if (isset($email) && $email) {
            $engagebay_array['email'] = sanitize_email($_POST['email']);
            $engagebay_array['name'] = sanitize_text_field($_POST['name']);
            $engagebay_array['website'] = sanitize_text_field($_POST['website']);
            $engagebay_array['password'] = sanitize_text_field($_POST['password']);
            $engagebay_array['command'] = sanitize_text_field($_POST['command']);
            $engagebay_array['timeZoneId'] = sanitize_text_field($_POST['timeZoneId']);
            $engagebay_array['source'] = 'WORDPRESS';

            $api_url = 'https://app.engagebay.com/rest/api/signup/signup-user';
            $request = wp_remote_post($api_url, array(
                'method' => 'post',
                'timeout' => 40,
                'httpversion' => '1.0',
                'blocking' => true,
                'body' => $engagebay_array,
                'headers' => array('ebwhitelist' => true),
            ));
            $result = wp_remote_retrieve_body($request);
            $resultString = $result;

            if (version_compare(PHP_VERSION, '5.4.0', '>=') && !(defined('JSON_C_VERSION') && PHP_INT_SIZE > 4)) {
                $result = json_decode($result, true, 512, JSON_BIGINT_AS_STRING);
            } else {
                $result = json_decode($result, true);
            }

            $domain = $result['domain_name'];
            $js_api = $result['api_key']['js_API_Key'];
            $rest_api = $result['api_key']['rest_API_Key'];
            $email = sanitize_email($_POST['email']);
            $password = sanitize_text_field($_POST['password']);
            if ($domain == '') {
                $errors = $resultString;
            } else {
                if (get_option('cf7ebi_settings_password') !== false) {
                    update_option('cf7ebi_settings_password', $password);
                } else {
                    add_option('cf7ebi_settings_password', $password, $deprecated, $autoload);
                }

                if (get_option('cf7ebi_settings_rest_api') !== false) {
                    update_option('cf7ebi_settings_rest_api', $rest_api);
                } else {
                    add_option('cf7ebi_settings_rest_api', $rest_api, $deprecated, $autoload);
                }

                if (get_option('cf7ebi_settings_domain') !== false) {
                    update_option('cf7ebi_settings_domain', $domain);
                } else {
                    add_option('cf7ebi_settings_domain', $domain, $deprecated, $autoload);
                }

                if (get_option('cf7ebi_settings_js_api') !== false) {
                    update_option('cf7ebi_settings_js_api', $js_api);
                } else {
                    add_option('cf7ebi_settings_js_api', $js_api, $deprecated, $autoload);
                }

                if (get_option('cf7ebi_settings_email') !== false) {
                    update_option('cf7ebi_settings_email', $email);
                } else {
                    add_option('cf7ebi_settings_email', $email, $deprecated, $autoload);
                }

                $sucesserrors = 'SucessFully Registered';
            }
        } ?>

        <div class="be-wrapper be-login">
            <div class="be-content">
                <div class="main-content container-fluid">
                    <div class="splash-container">
                        <div class="panel">
                            <div class="panel-heading">

                                <?php echo "<img src='" . cf7ebi_root_url('assets/images/engagebay.png', __FILE__) . "' width='170' title='Enage bay logo'/>"; ?>
                                <h4>
                                    Get Started for <strong>FREE</strong>
                                </h4>
                            </div>
                            <?php if (!isset($sucesserrors)) {
                            ?>
                                <div class="panel-body">
                                    <?php
                                    if (isset($errors)) {
                                    ?>
                                        <div class="alert alert-danger">
                                            <strong><?php echo $errors; ?> </strong>
                                            <?php echo $result; ?>
                                        </div>
                                    <?php
                                    } ?>
                                    <form id="loginForm" name="loginForm" method="POST">

                                        <input type="hidden" name="command" value="signup">
                                        <input type="hidden" name="timeZoneId" value="330">
                                        <input type="hidden" name="referral_user_id" value="">
                                        <div class="form-group">
                                            <div class="">
                                                <input class="form-control" type="name" name="name" autocomplete="name" placeholder="Name" title="Name should be between 1-30 characters in length. Both letters and numbers are allowed but it should start with a letter. Cannot contain special characters." minlength="1" maxlength="30" required="" pattern="^[a-zA-Z][a-zA-Z0-9 ]{1,30}$">
                                            </div>
                                        </div>
                                        <div class="form-group">
                                            <div class="position-relative">
                                                <input class="form-control user-email-field" id="register_email" type="email" name="email" required autocapitalize="off" minlength="6" maxlength="50" placeholder="Work Email" oninvalid="set_custom_validate(this);" oninput="reset_custom_validate(this);" pattern="^.+@((?!gmail.com)(?!yahoo.com)(?!yahoo.in)(?!hotmail.com)(?!fastmail.com).)+\..+$" data-pattern-mismatch-error="Please use your business email address to sign up" value="">
                                                <span id="validEmail" class="validation-result">
                                                </span>
                                            </div>
                                        </div>
                                        <div class="form-group">
                                            <div class="">
                                                <input class="form-control" type="url" name="website" autocomplete="url" placeholder="Website URL" value="" minlength="3" maxlength="50" required="" pattern="^(https?://)?([a-zA-Z0-9]([a-zA-Z0-9\-]{0,61}[a-zA-Z0-9])?\.)+[a-zA-Z]{2,6}$">
                                            </div>
                                        </div>
                                        <div class="form-group">
                                            <div class="">
                                                <input class="form-control" type="password" name="password" pattern=".{4,20}" autocomplete="off" placeholder="Password" title="Enter at least 4 characters." required="">
                                            </div>
                                        </div>
                                        <div class="form-group">
                                            <div class="xs-pt-20 md-pt-20">
                                                <div class="login-submit xs-m-0 md-p-0">
                                                    <button class="btn btn-success btn-xl" type="submit">
                                                        SIGN UP
                                                        <?php echo "<img src='" . cf7ebi_root_url('assets/images/loader.gif', __FILE__) . "' 
			                       style='display: none;' id='gif' title='Landing Pages'/>"; ?>
                                                    </button>
                                                </div>
                                            </div>
                                        </div>
                                    </form>
                                </div>

                        </div>
                        <div class="splash-footer">
                            Forgot <a href="https://app.engagebay.com/forgot-password" target="_blank" class="text-info">Password?</a>
                        </div>
                        <div class="splash-footer alert">
                            Already have an account? <a href="?page=engagebay-cf7-settings"><?php _e('Sign In', 'sandbox'); ?></a>
                        </div>
                    <?php
                            } else {
                    ?>
                        <div class="panel-body">
                            <div class="alert alert-success fade in alert-dismissible">
                                <strong>Successfully!</strong> Registered.
                            </div>
                        </div>
                    <?php
                                cf7ebi_engagebay_refresh();
                            } ?>

                    </div>
                </div>
            </div>
        </div>
<?php
    }

    function cf7ebi_admin_panel($panels)
    {
        $new_page = array(
            'engagebay-forms-integration-addon' => array(
                'title' => __('EngageBay Integration', 'contact-form-7'),
                'callback' => 'cf7ebi_admin_panel_content',
            ),
        );

        $panels = array_merge($panels, $new_page);

        return $panels;
    }
    add_filter('wpcf7_editor_panels', 'cf7ebi_admin_panel');

    function cf7ebi_admin_panel_content($cf7)
    {
        $engagebay_domain = get_option('cf7ebi_settings_domain');
        $engagebay_email = get_option('cf7ebi_settings_email');
        $engagebay_rest_api_key = get_option('cf7ebi_settings_rest_api');

        $post_id = $cf7->id();
        $cf7_fields = $cf7->scan_form_tags();

        $template = cf7ebi_get_view_template('form-fields.tpl.php');

        $engagebayFields = array(
            array('name' => 'name', 'label' => 'First name*', 'is_required' => true, 'type' => 'SYSTEM'),
            array('name' => 'last_name', 'label' => 'Last name', 'is_required' => true, 'type' => 'SYSTEM'),
            array('name' => 'company', 'label' => 'Company', 'is_required' => false, 'type' => 'SYSTEM'),
            array('name' => 'email', 'label' => 'Email', 'is_required' => false, 'type' => 'SYSTEM'),
            array('name' => 'phone', 'label' => 'Phone', 'is_required' => false, 'type' => 'SYSTEM'),
            array('name' => 'country', 'label' => 'Country', 'is_required' => false, 'type' => 'SYSTEM'),
            array('name' => 'address', 'label' => 'Address', 'field_type' => 'SYSTEM_ADDRESS', 'is_required' => false, 'type' => 'SYSTEM'),
            array('name' => 'city', 'label' => 'City', 'field_type' => 'SYSTEM_ADDRESS', 'is_required' => false, 'type' => 'SYSTEM'),
            array('name' => 'state', 'label' => 'State', 'field_type' => 'SYSTEM_ADDRESS', 'is_required' => false, 'type' => 'SYSTEM'),
            array('name' => 'zip', 'label' => 'Zip', 'field_type' => 'SYSTEM_ADDRESS', 'is_required' => false, 'type' => 'SYSTEM')
        );

        $engagebay_rest_url = 'https://app.engagebay.com/dev/api/panel/customfields/CONTACT?page_size=200';
        $headers = array(
            'Authorization' => $engagebay_rest_api_key,
            'Content-type' => 'application/json',
            'Accept' => 'application/json',
            'ebwhitelist' => true
        );

        $args = array(
            'timeout' => 120,
            'sslverify' => false,
            'headers' => $headers,
        );

        $request = wp_remote_get($engagebay_rest_url, $args);
        $customFields = wp_remote_retrieve_body($request);

        if ($customFields) {
            $customFields = json_decode($customFields, true);
            if (is_array($customFields) && count($customFields) > 0) {
                foreach ($customFields as $customField) {
                    if (isset($customField['field_label'])) {
                        $engagebayFields[] = array(
                            'name' => $customField['field_name'],
                            'label' => $customField['field_label'],
                            'field_type' => $customField['field_type'],
                            'is_required' => (bool) $customField['is_required'],
                            'type' => 'CUSTOM',
                        );
                    }
                }
            }
        }

        update_option('cf7ebi_engagebay_fields', $engagebayFields);

        $html = '';
        foreach ($engagebayFields as $engagebayField) {
            $html .= '<tr>';
            $html .= '<td>' . $engagebayField['label'] . '</td>';
            $html .= '<td><select name="cf7ebi_mapped_' . $engagebayField['name'] . '">';
            foreach ($cf7_fields as $cf7_field) {
                $selected = '';
                if (get_post_meta($post_id, '_cf7ebi_mapped_' . $engagebayField['name'], true) == $cf7_field['name']) {
                    $selected = 'selected';
                }
                $html .= '<option value="' . $cf7_field['name'] . '" ' . $selected . '>' . $cf7_field['name'] . '</option>';
            }
            $html .= '</select></td>';
            $html .= '</tr>';
        }

        $search_replace = array(
            '{cf7ebi_engagebay_fields}' => $html,
        );

        $search = array_keys($search_replace);
        $replace = array_values($search_replace);

        $form_fields_template = str_replace($search, $replace, $template);

        $template = cf7ebi_get_view_template('ui-tabs-panel.tpl.php');

        $cf7ebi_map_enabled = get_post_meta($post_id, '_cf7ebi_enabled', true);

        $search_replace = array(
            '{enabled}' => get_post_meta($post_id, '_cf7ebi_enabled', true) == 1 ? 'checked' : '',
            '{cf7ebi_form_tags}' => get_post_meta($post_id, '_cf7ebi_form_tags', true),
            '{form_fields_html}' => $form_fields_template,
            '{plugin_url}' => cf7ebi_root_url(),
            '{site_url}' => get_site_url(),
        );

        $search = array_keys($search_replace);
        $replace = array_values($search_replace);

        $template_content = str_replace($search, $replace, $template);

        echo $template_content;
    }

    /**
     * Sanitize custom field names, return value is used as a key.
     */
    function clean($string)
    {
        $string = str_replace(' ', '-', $string); // Replaces all spaces with hyphens.
        $string = preg_replace('/[^A-Za-z0-9\-]/', '', $string); // Removes special chars.
        return preg_replace('/-+/', '-', $string); // Replaces multiple hyphens with single one.
    }

    function cf7ebi_get_view_template($template_name)
    {
        $template_content = false;
        $template_path = CF7EBI_VIEWS_DIR . $template_name;

        if (file_exists($template_path)) {
            $search_replace = array(
                "<?php if(!defined( 'ABSPATH')) exit; ?>" => '',
                '{plugin_url}' => cf7ebi_root_url(),
                '{site_url}' => get_site_url(),
            );

            $search = array_keys($search_replace);
            $replace = array_values($search_replace);

            $template_content = str_replace($search, $replace, file_get_contents($template_path));
        }

        return $template_content;
    }

    function cf7ebi_admin_save_form($cf7)
    {
        $post_id = sanitize_text_field($_GET['post']);

        if (isset($_POST['cf7ebi_enabled'])) {
            $cf7ebi_enabled = filter_var($_POST['cf7ebi_enabled'], FILTER_SANITIZE_NUMBER_INT);
            update_post_meta($post_id, '_cf7ebi_enabled', $cf7ebi_enabled);
        }

        $engagebayFields = get_option('cf7ebi_engagebay_fields');

        foreach ($engagebayFields as $engagebayField) {
            update_post_meta($post_id, '_cf7ebi_mapped_' . $engagebayField['name'], sanitize_text_field($_POST['cf7ebi_mapped_' . $engagebayField['name']]));
        }

        update_post_meta($post_id, '_cf7ebi_form_tags', sanitize_text_field($_POST['cf7ebi_form_tags']));
    }
    add_action('wpcf7_save_contact_form', 'cf7ebi_admin_save_form');

    function cf7ebi_frontend_submit_form($wpcf7_data)
    {
        $post_id = $wpcf7_data->id;
        $enabled = get_post_meta($post_id, '_cf7ebi_enabled', true);

        if ($enabled == 1) {
            $engagebay_domain = get_option('cf7ebi_settings_domain');
            $engagebay_email = get_option('cf7ebi_settings_email');
            $engagebay_rest_api_key = get_option('cf7ebi_settings_rest_api');
            $engagebay_fields = get_option('cf7ebi_engagebay_fields');

            foreach ($engagebay_fields as $key => $engagebay_field) {
                $post_name = get_post_meta($post_id, '_cf7ebi_mapped_' . $engagebay_field['name'], true);
                $sanitized_post_name = sanitize_text_field($_POST[$post_name]);
                if (!isset($sanitized_post_name)) {
                    $value = '';
                } else {
                    $value = $sanitized_post_name;
                }

                if (isset($engagebay_field['field_type']) && $engagebay_field['field_type'] == 'MULTICHECKBOX') {
                    if (is_array($value)) {
                        $value = preg_replace('/,\s+/', ',', implode(',', $value));
                    }

                    $contact_properties[] = array(
                        'name' => $engagebay_field['name'],
                        'value' => $value,
                        'field_type' => 'MULTICHECKBOX',
                        'is_searchable' => 'false',
                        'type' => $engagebay_field['type'],
                    );
                    continue;
                }

                if (isset($engagebay_field['field_type']) && $engagebay_field['field_type'] == 'CHECKBOX') {
                    $contact_properties[] = array(
                        'name' => $engagebay_field['name'],
                        'value' => $value,
                        'field_type' => 'CHECKBOX',
                        'is_searchable' => 'true',
                        'type' => $engagebay_field['type'],
                    );
                    continue;
                }

                if ($engagebay_field['type'] == 'SYSTEM' && $engagebay_field['name'] == 'company') {
                    $company_name = $value;
                    continue;
                }

                if ($engagebay_field['type'] == 'SYSTEM' && $engagebay_field['name'] == 'country') {
                    $country = $value;
                    continue;
                }

                if ($engagebay_field['field_type'] == 'SYSTEM_ADDRESS') {
                    if (empty($value)) {
                        continue;
                    }
                    if ($engagebay_field['name'] == 'address') {
                        $address = $value;
                    }
                    if ($engagebay_field['name'] == 'city') {
                        $city = $value;
                    }
                    if ($engagebay_field['name'] == 'state') {
                        $state = $value;
                    }
                    if ($engagebay_field['name'] == 'zip') {
                        $zip = $value;
                    }
                    continue;
                }

                if (isset($engagebay_field['field_type']) && $engagebay_field['field_type'] == 'DATE') {
                    $contact_properties[] = array(
                        'name' => $engagebay_field['name'],
                        'value' => strtotime($value) == '' ? '' : strtotime($value),
                        'field_type' => 'DATE',
                        'is_searchable' => 'true',
                        'type' => $engagebay_field['type'],
                    );
                    continue;
                }

                if (isset($engagebay_field['field_type']) && $engagebay_field['field_type'] == 'URL') {
                    $contact_properties[] = array(
                        'name' => $engagebay_field['name'],
                        'value' => $value,
                        'field_type' => 'URL',
                        'is_searchable' => 'true',
                        'type' => $engagebay_field['type'],
                    );
                    continue;
                }

                if (isset($engagebay_field['field_type']) && $engagebay_field['field_type'] == 'NUMBER') {
                    $contact_properties[] = array(
                        'name' => $engagebay_field['name'],
                        'value' => $value,
                        'field_type' => 'NUMBER',
                        'is_searchable' => 'true',
                        'type' => $engagebay_field['type'],
                    );
                    continue;
                }

                if (isset($engagebay_field['field_type']) && $engagebay_field['field_type'] == 'LIST') {
                    if (is_array($value)) {
                        $value = $value[0];
                    }
                    $contact_properties[] = array(
                        'name' => $engagebay_field['name'],
                        'value' => $value,
                        'field_type' => 'LIST',
                        'is_searchable' => 'true',
                        'type' => $engagebay_field['type'],
                    );
                    continue;
                }

                if (isset($engagebay_field['field_type']) && $engagebay_field['field_type'] == 'TEXTAREA') {
                    $contact_properties[] = array(
                        'name' => $engagebay_field['name'],
                        'value' => $value,
                        'field_type' => 'TEXTAREA',
                        'is_searchable' => 'true',
                        'type' => $engagebay_field['type'],
                    );
                    continue;
                }

                if ($engagebay_field['name'] == 'phone') {
                    $contact_properties[] = array(
                        'name' => $engagebay_field['name'],
                        'value' => $value,
                        'field_type' => 'TEXT',
                        'is_searchable' => 'false',
                        'subtype' => 'work',
                        'type' => $engagebay_field['type'],
                    );
                    continue;
                }

                $contact_properties[] = array(
                    'name' => $engagebay_field['name'],
                    'value' => $value,
                    'field_type' => 'TEXT',
                    'is_searchable' => 'false',
                    'type' => $engagebay_field['type'],
                );

                if (isset($address)) {
                    $contact_properties[] = array(
                        'name' => 'address',
                        'value' => json_encode([
                            'address'   => isset($address) ? $address : '',
                            'city'      => isset($city) ? $city : '',
                            'state'     => isset($state) ? $state : '',
                            'country'   => isset($country) ? $country : '',
                            'zip'       => isset($zip) ? $zip: ''
                        ], JSON_PRETTY_PRINT),
                        'field_type'    => 'TEXT',
                        'is_searchable' => 'true',
                        'type' => $engagebay_field['type'],
                    );
                }

                if ($engagebay_field['name'] == 'email') {
                    $email_in_form = sanitize_email($_POST[$post_name]);
                }
            }

            $all_tags = get_post_meta($post_id, '_cf7ebi_form_tags', true);
            $split_tags = explode(',', $all_tags);

            foreach ($split_tags as $tag) {
                $tags[] = array(
                    'tag' => $tag,
                );
            }

            $engagebay_rest_url = 'https://app.engagebay.com/dev/api/panel/subscribers/getByEmail/' . $email_in_form;
            $headers = array(
                'Authorization' => $engagebay_rest_api_key,
                'Content-type' => 'application/json',
                'Accept' => 'application/json',
                'ebwhitelist' => true
            );

            $args_get = array(
                'method' => 'GET',
                'timeout' => 120,
                'sslverify' => false,
                'headers' => $headers,
            );

            $request = wp_remote_post($engagebay_rest_url, $args_get);

            $search_result = wp_remote_retrieve_body($request);
            if (version_compare(PHP_VERSION, '5.4.0', '>=') && !(defined('JSON_C_VERSION') && PHP_INT_SIZE > 4)) {
                $search_result = json_decode($search_result, true, 512, JSON_BIGINT_AS_STRING);
            } else {
                $search_result = json_decode($search_result, true);
            }

            $final_contact_properties = [];
            foreach ($contact_properties as $contact_property) {
                if ($contact_property['value'] != '') {
                    array_push($final_contact_properties, $contact_property);
                }
            }

            $finalData = array('status' => 'CONFIRMED', 'score' => 10, 'properties' => $final_contact_properties, 'tags' => $tags);
            
            if (isset($company_name)) {
                $finalData['company_name'] = $company_name;
            }

            $engagebay_rest_url = 'https://app.engagebay.com/dev/api/panel/subscribers/subscriber';
            $headers = array(
                'Authorization' => $engagebay_rest_api_key,
                'Content-type' => 'application/json',
                'Accept' => 'application/json',
                'ebwhitelist' => true
            );

            if (isset($search_result) && is_array($search_result) && count($search_result) > 0) {
                $engagebay_rest_url = 'https://app.engagebay.com/dev/api/panel/subscribers/update-partial';
                $finalData['id'] = $search_result->id;
                $args_put = array(
                    'method' => 'PUT',
                    'timeout' => 120,
                    'sslverify' => false,
                    'headers' => $headers,
                    'body' => json_encode($finalData),
                );
                $response = wp_remote_request($engagebay_rest_url, $args_put);
            } else {
                $args_post = array(
                    'method' => 'POST',
                    'timeout' => 120,
                    'sslverify' => false,
                    'headers' => $headers,
                    'body' => json_encode($finalData),
                );
                wp_remote_post($engagebay_rest_url, $args_post);
            }
        }
    }
    add_action('wpcf7_before_send_mail', 'cf7ebi_frontend_submit_form');

    // plugin deactivation
    register_deactivation_hook(__FILE__, 'cf7ebi_deactivate');
    function cf7ebi_deactivate()
    {
        delete_option('cf7ebi_engagebay_fields');
        delete_option('cf7ebi_settings_password');
        delete_option('cf7ebi_settings_rest_api');
        delete_option('cf7ebi_settings_domain');
        delete_option('cf7ebi_settings_js_api');
        delete_option('cf7ebi_settings_email');
    }
}
